@@ -736,7 +736,7 @@ public class Parser {
       importedStatementList.add("java.lang.*");
       
       currentLine = 1;
-      
+
       char[] source = loadFile(file, encoding);
 
       try {
@@ -828,7 +828,7 @@ public class Parser {
             if (null == currentPackageName ||
                 !currentPackageName.equals(expectedPackageName)) {
 
-               Main.getRootDoc().printWarning("Ignoring file " + currentFile + ": (wrong package)");
+               Main.getRootDoc().printWarning("Ignoring file " + currentFile + ": (wrong package, " + currentPackageName + "!=" + expectedPackageName + ")");
                throw new IgnoredFileParseException();
             }
          }
@@ -837,7 +837,7 @@ public class Parser {
             currentPackage = Main.getRootDoc().findOrCreatePackageDoc(currentPackageName);
          }
          else {
-            currentPackage = PackageDocImpl.DEFAULT_PACKAGE;
+            currentPackage = Main.getRootDoc().findOrCreatePackageDoc("");
          }
       }
 
